<?php declare( strict_types = 1 );
/*
 * (c) shopware AG <info@shopware.com>
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace GPWebPayGpe\AdapterPlugin\Service;

use Psr\Log\LoggerInterface;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Checkout\Payment\Cart\AsyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\AsynchronousPaymentHandlerInterface;
use Shopware\Core\Checkout\Payment\Exception\CustomerCanceledAsyncPaymentException;
use Shopware\Core\Checkout\Payment\Exception\InvalidTransactionException;
use Shopware\Core\Checkout\Payment\Exception\PaymentProcessException;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\MultiFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\NotFilter;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\Migration\Migration1602494495SetUsersAsAdmins;

use Shopware\Core\System\SalesChannel\SalesChannelContext;
use GPWebPayGpe\AdapterPlugin\Service\AdapterPayment;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\HttpFoundation\Response;

class AdapterFinalizeController extends AbstractController {

	/**
	 * @var AdapterPayment
	 */
	private $adapterPayment;

	/**
	 * @var EntityRepositoryInterface
	 */
	private $orderTransactionRepo;

	/**
	 * @var AsynchronousPaymentHandlerInterface
	 */
	private $paymentHandler;

	/**
	 * @var OrderTransactionStateHandler
	 */
	private $transactionStateHandler;

	/**
	 * @var RouterInterface
	 */
	private $router;


	public function __construct(
		AdapterPayment $adapterPayment,
		EntityRepositoryInterface $orderTransactionRepo,
		OrderTransactionStateHandler $transactionStateHandler,
		RouterInterface $router
	) {
		$this->adapterPayment          = $adapterPayment;
		$this->orderTransactionRepo    = $orderTransactionRepo;
		$this->transactionStateHandler = $transactionStateHandler;
		$this->router                  = $router;
	}

	/**
	 * @RouteScope(scopes={"storefront"})
	 * @Route("/adapter/finalize", defaults={"csrf_protected"=false, "auth_required"=false}, name="gpwebpay_finalize")
	 */
	public function finalize(
		Request $request,
		SalesChannelContext $salesChannelContext
	): Response {

		$orderTransactionId = $request->get( 'order_transaction_id' );
		$criteria           = new Criteria( [ $orderTransactionId ] );
		$criteria->addAssociation( 'order' );
		$criteria->addAssociation( 'paymentMethod' );

		/**
		 * @var null|OrderTransactionEntity $orderTransaction
		 */
		$orderTransaction = $this->orderTransactionRepo->search( $criteria, Context::createDefaultContext() )->first();
		$order            = $orderTransaction->getOrder();
		$transaction      = new AsyncPaymentTransactionStruct( $orderTransaction, $order, '' );
		$salesChannelId   = $order->getSalesChannelId();

		$orderReplyStatus = $this->adapterPayment->uniModul->gatewayReceiveReply( $this->adapterPayment->getConfig( 'language', $salesChannelId ) );
		$finishUrl        = $orderReplyStatus->uniAdapterData['adapter_return_url'];
		$context          = $salesChannelContext->getContext();
		$orderId          = $order->getId();

		try {

			$this->saveTransactionData( $transaction, $orderReplyStatus, $context );
			$this->adapterPayment->finalize( $transaction, $request, $salesChannelContext );

		} catch ( PaymentProcessException $paymentProcessException ) {
			$finishUrl = $this->redirectToConfirmPageWorkflow(
				$paymentProcessException,
				$context,
				$orderId
			);
		}

		return new RedirectResponse( $finishUrl );

	}


	private function redirectToConfirmPageWorkflow(
		PaymentProcessException $paymentProcessException,
		Context $context,
		string $orderId
	): string {
		$errorUrl = $this->router->generate( 'frontend.account.edit-order.page', [ 'orderId' => $orderId ] );

		if ( $paymentProcessException instanceof CustomerCanceledAsyncPaymentException ) {
			$urlQuery = \parse_url( $errorUrl, PHP_URL_QUERY ) ? '&' : '?';

			return \sprintf( '%s%serror-code=%s', $errorUrl, $urlQuery, $paymentProcessException->getErrorCode() );
		}

		$urlQuery = \parse_url( $errorUrl, PHP_URL_QUERY ) ? '&' : '?';

		return \sprintf( '%s%serror-code=%s', $errorUrl, $urlQuery, $paymentProcessException->getErrorCode() );
	}



	/**
	 * @param AsyncPaymentTransactionStruct $transaction
	 * @param $orderReplyStatus
	 * @param $context
	 */
	public function saveTransactionData(AsyncPaymentTransactionStruct $transaction, $orderReplyStatus,  $context): void
	{
		$data = [
			'gpwebpay_transaction_id' => $orderReplyStatus->gwOrderNumber,
		];

		$customFields = $transaction->getOrderTransaction()->getCustomFields() ?? [];
		$customFields = array_merge($customFields, $data);
		$update = [
			'id' => $transaction->getOrderTransaction()->getId(),
			'customFields' => $customFields,
		];
		$transaction->getOrderTransaction()->setCustomFields($customFields);
		$this->orderTransactionRepo->update([$update], $context);
	}




	/*
    private function isAtLeastShopware6330(): bool
    {
        return \class_exists(Migration1602494495SetUsersAsAdmins::class);
    }
	*/
}
